#include <iostream>
#include <vector>
#include <boost/gil/gil_all.hpp>
#include <boost/gil/extension/io/jpeg_io.hpp>
#include "util.h"
#include <boost/bind.hpp>
#include <boost/timer.hpp>
#include <boost/format.hpp>
#include <boost/ref.hpp>

#include "poselet_api.h"
#include "gil_draw.h"

// Requires imdebug which is only available on Windows
// http://www.billbaxter.com/projects/imdebug/
#ifdef USE_IMDEBUG
#include "gil_imdebug.h"
#endif

#include <sys/types.h>

#include <stdio.h>
#include <fstream>
#include <sstream>
#include <string>
using namespace std;
using namespace boost::gil;
using namespace poselet_api;


template <typename Value>
void append_hit(const Value& oh, vector<Value>& hits) {
  hits.push_back(oh);
}


#ifdef USE_IMDEBUG
template <typename View, typename Hit, typename Color>
void draw_hit(const View& v, const Hit& hit, const Color& c, int thickness=1) {
  typename View::value_type color;
  boost::gil::color_convert(c,color);
  int_bounds b(hit.x0,hit.y0,hit.width,hit.height);
  draw_bounds(v,color,b,thickness);

  render_text(v, b._min+int_point(thickness,thickness),
              str(boost::format("%4.2f") % hit.score), c, 1, 1);
}
#endif

bool file_exists(const char *filename)
{
  ifstream ifile(filename);
  return !!ifile;
}

int main( int argc, char* argv[] ) {
  if (argc<3) {
    cout << "Usage: "<<argv[0]
         <<" <detector.xml> <image.jpg>"<<endl;
    return 0;
  }
  if (!file_exists(argv[1])) {
    cout << "Cannot open model file "<<argv[1]<<endl;
    return 0;
  }
  if (!file_exists(argv[2])) {
    cout << "Cannot open image file "<<argv[2]<<endl;
    return 0;
  }

  bgr8_image_t img;
  jpeg_read_image(argv[2],img);

  string filename(argv[1]);
  size_t nch = filename.find_last_of("/\\");
  string dir, modelname;
  if (nch == string::npos) {
    dir = ".";
    nch = -1;
  } else {
    dir = filename.substr(0, nch);
  }
  modelname = filename.substr(nch+1,filename.length()-nch-5);
  cout << "Dir: "<<dir<<" model:"<<modelname<<endl;

  InitDetector(dir.c_str(), modelname.c_str(), true);

  boost::timer t;

  Image img_proxy(
    img.width(), img.height(), const_view(img).pixels().row_size(),
    Image::k8Bit, Image::kRGB, interleaved_view_get_raw_data(const_view(img)));

  vector<ObjectHit> object_hits;
  vector<PoseletHit> poselet_hits;

  RunDetector(img_proxy, boost::bind(append_hit<PoseletHit>, _1,
    boost::ref(poselet_hits)), boost::bind(append_hit<ObjectHit>, _1,
    boost::ref(object_hits)), true, 0, false);

  double scoresum=0;
  for (size_t i=0; i<object_hits.size(); ++i) {
#ifdef USE_IMDEBUG
    if (i<2)
      draw_hit(view(img), object_hits[i], rgb8_pixel_t(255,0,0), 3);
#endif
    scoresum+=object_hits[i].score;
  }
#ifdef USE_IMDEBUG
  debug_display_view(view(img));
#endif

  cout << "Found "<<object_hits.size()<<" objects with scoresum "<<scoresum
       << " in "<<t.elapsed()<<" sec."<<endl;
  return 0;
}
